'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var util = require('util');
var Core = require('../core');
var Schemas = require('../Schemas');

/**
*
* Base Transport
*
*/

var BaseTransport = function (_Core) {
  _inherits(BaseTransport, _Core);

  function BaseTransport(options) {
    _classCallCheck(this, BaseTransport);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(BaseTransport).call(this, options));

    _this._subscription = {};

    if (_this.options.schemas) {
      _this.setSchemas(_this.options.schemas);
    }

    // The clients can be used both ways, but we need to
    // know what their role is, normal role for a client
    // is being the runtime, runtime will be refered to as `server` role
    // a UI or whatever commander will be refered to as `client` role
    // default role is `server`
    if (_this.options.role === 'client') {
      _this.receiveMode = 'output';
      _this.replyMode = 'input';
      if (!_this.options.secret) {
        throw Error('In client mode options requires a secret');
      }
    } else {
      _this.receiveMode = 'input';
      _this.replyMode = 'output';
    }

    _this.connections = {};
    return _this;
  }

  _createClass(BaseTransport, [{
    key: 'sendAll',
    value: function sendAll(protocol, command, data, graphId) {
      var self = this;

      // data would contain `graph`, topic tells us we need graph.
      // TODO: merge this with this.connections.
      graphId = graphId || data.graph;
      var topic = 'graph';
      if (this._subscription.hasOwnProperty(topic)) {
        if (this._subscription[topic].hasOwnProperty(graphId)) {
          this._subscription[topic][graphId].forEach(function (conn) {
            if (command === 'error') {
              self.sendError(protocol, data, conn);
            } else {
              self.$send({
                protocol: protocol,
                command: command,
                payload: data
              }, conn);
            }
          });
        } else {
          // For now just do not send back.
          // subgraphs are not subscribed, so just do not send info about them
          // e.g. add link will also trigger for subcomponents beneath.
          // The question is, do we need to know about those additions at all.
          // I think not..
          console.log('protocol:' + protocol, 'topic:' + topic, data);
          this.logger.warn('BaseTransport', util.format('`%s` property `%s` not found', topic, graphId));
        }
      }
    }

    /**
     *
     * Internal Receive
     *
     * Emit's the validated input for processing.
     *
     */

  }, {
    key: 'receive',
    value: function receive(data, conn) {
      this.logger.debug('BaseTransport', 'Receive:', data);

      var ret = this.validate(this.receiveMode, data.protocol, data.command, data.payload);

      if (util.isError(ret)) {
        var msg = {
          message: ret.toString(),
          graph: data.payload.graph
        };
        this.logger.error('BaseTransport', 'Error:', msg);
        this.sendError(data.protocol || 'runtime', msg, conn);
        // this.$send(msg, conn)
      } else {
        // this should be generic
        // Graph subscription
        if (data.protocol === 'graph' && data.command === 'clear') {
          this.subscribe(conn, 'graph', data.payload.id);
        } else if (data.protocol === 'network' && data.command === 'getstatus') {
          this.subscribe(conn, 'graph', data.payload.graph);
        }

        this.emit(data.protocol, data, conn);
      }
    }

    /**
     *
     * Validates incomming and outgoing messages.
     *
     */

  }, {
    key: 'validate',
    value: function validate(type, protocol, command, data) {
      if (!this.protocolSchema) {
        throw Error('Schemas required');
      }
      return this.protocolSchema.validate(type, protocol, command, data);
    }
  }, {
    key: 'setSchemas',
    value: function setSchemas(schemas) {
      this.protocolSchema = new Schemas();
      this.protocolSchema.load(schemas);
    }

    /**
     *
     * Generic method to send out the errors
     * Can be used during extention to intercept all errors.
     *
     * @param protocol
     * @param payload
     * @param conn
     */

  }, {
    key: 'sendError',
    value: function sendError(protocol, payload, conn) {
      var msg = {
        protocol: protocol,
        command: 'error',
        payload: payload
      };
      this.$send(msg, conn);
    }

    /**
     *
     * Internal reply method.
     *
     * Does the generic stuff then uses the send method of the transport.
     *
     */

  }, {
    key: 'reply',
    value: function reply(protocol, command, data, conn) {
      if (data === undefined) {
        throw Error('data must not be undefined');
      }

      /*
       if (data.command === 'error' && util.isError(data.payload.message)) {
       data.payload.message.toString()
       }
       */
      if (data.command === 'error') {
        this.sendError(data.protocol, data.payload, conn);
      } else {
        // too many messages
        // solve this with something like debug: true
        if (command !== 'component') {
          this.logger.debug('BaseTransport', 'Respond', protocol + ':' + command, data);
        }

        var ret = this.validate(this.replyMode, protocol, command, data);
        if (util.isError(ret)) {
          if (this.options.bail) {
            throw ret;
          } else {
            this.logger.warn('BaseTransport', ret);
            this.sendError(protocol, ret, conn);
            // this.$send(msg, conn)
          }
        } else {
          var msg = {
            protocol: protocol,
            command: command,
            payload: data
          };
          this.$send(msg, conn);
        }
      }
    }
  }, {
    key: '$send',
    value: function $send() /* data, conn */{
      throw Error('Send method not implemented');
    }
  }, {
    key: 'getAll',
    value: function getAll(protocol) {
      return this.connections[protocol || 'default'] || [];
    }
  }, {
    key: 'register',
    value: function register(conn, protocol) {
      protocol = protocol || 'default';
      if (!this.connections.hasOwnProperty(protocol)) {
        this.connections[protocol] = [];
      }
      this.connections[protocol].push(conn);
      this.emit('register', conn);
    }
  }, {
    key: 'isRegistered',
    value: function isRegistered(conn, protocol) {
      protocol = protocol || 'default';
      return this.connections[protocol] && this.connections[protocol].indexOf(conn) >= 0;
    }
  }, {
    key: 'unregister',
    value: function unregister(conn, protocol) {
      protocol = protocol || 'default';
      if (this.connections[protocol].indexOf(conn) === -1) {
        return;
      }

      this.connections[protocol].splice(this.connections[protocol].indexOf(conn), 1);
      this.emit('unregister', conn);
      // unsubscribe from everything
    }

    /**
     *
     * Used to subscribe a connection to a key value pair.
     *
     * 'graph': [<id>, <id> ]
     *
     * transport.subscribe(conn, 'graph', value)
     *
     * This method is called by handlers so when the monitor does a sendAll
     * the transport is able to determine who is all.
     *
     * Subscribe is done during graph creation.
     *
     */

  }, {
    key: 'subscribe',
    value: function subscribe(conn, key, value) {
      if (!this._subscription.hasOwnProperty(key)) {
        this._subscription[key] = {};
      }

      if (!this._subscription[key].hasOwnProperty(value)) {
        this._subscription[key][value] = [];
      }

      if (this._subscription[key][value].indexOf(conn) === -1) {
        this._subscription[key][value].push(conn);
      }

      this.emit('subscribe', conn, key, value);

      this.logger.info('BaseTransport', 'Connection subscribed to %s %s', key, value);
    }
  }, {
    key: 'isSubscribed',
    value: function isSubscribed(conn, key, value) {
      if (!this._subscription.hasOwnProperty(key)) {
        return false;
      }
      if (!this._subscription[key].hasOwnProperty(value)) {
        return false;
      }
      return this._subscription[key][value].indexOf(conn) >= 0;
    }
  }, {
    key: 'unsubscribe',
    value: function unsubscribe(conn, key, value) {
      if (this._subscription.hasOwnProperty(key) && this._subscription[key].hasOwnProperty(value)) {
        this._subscription[key][value].splice(this._subscription[key][value].indexOf(conn), 1);

        this.emit('unsubscribe', conn, key, value);

        this.logger.info('BaseTransport', 'Connection unsubscribed from %s %s', key, value);
      }
    }
  }, {
    key: 'setSecret',
    value: function setSecret(data) {
      if (this.options.secret) {
        if (!data.payload) {
          data.payload = {};
        }
        data.payload.secret = this.options.secret;
      }
    }
  }]);

  return BaseTransport;
}(Core);

module.exports = BaseTransport;