'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BaseTransport = require('./base');
var httpServer = require('http');
var WebSocketServer = require('websocket').server;

/**
*
* Websocket Transport
*
* Usage:
*
*  const nm = new NetworkMonitor({
*    transport: new WebSocketTransport({ logger: console }),
*    logger: console
*  })
*  nm.monitor(ioHandler)
*
* @params {Server} http HTTP Server
* @params {Object} options Options
* @params {String} options.catchExceptions Send all exceptions to UI.
* @params {String} options.type runtime type chix
*/

var WebSocketTransport = function (_BaseTransport) {
  _inherits(WebSocketTransport, _BaseTransport);

  // constructor(http, options) {
  function WebSocketTransport(options, http) {
    _classCallCheck(this, WebSocketTransport);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(WebSocketTransport).call(this, options));
    // TODO: http as first param is a bit weird


    var self = _this;

    if (!http) {
      http = httpServer.createServer(function (req, res) {
        res.writeHead(200, {
          'Content-Type': 'application/json'
        });
        res.end(JSON.stringify({
          connections: _this.connections.length,
          pendingRequests: _this.pendingRequests.length
        }));
      });
      self.logger.info('wsServer', 'created http server');
    }

    _this.options = options;

    _this.ws = new WebSocketServer({
      httpServer: http
    });

    _this.ws.on('request', function (req) {
      var conn = void 0;
      try {
        conn = req.accept('noflo', req.origin);
        self.logger.info('wsServer', 'connection accepted', conn.protocol);
        console.log(conn.protocol);
      } catch (e) {
        console.log(e);
        self.logger.error('wsServer', e);
        return;
      }

      self.register(conn);

      conn.on('message', function (message) {
        if (message.type === 'utf8') {
          var payload = JSON.parse(message.utf8Data);
          self.receive(payload, conn);
        } else {
          self.logger.warn('wsServer', 'unknown message type', message);
        }
      });

      conn.on('close', function (message) {
        self.logger.info('wsServer', 'connection close', message);

        self.unregister(conn);
      });
    });

    if (options.catchExceptions) {
      process.on('uncaughtException', function (err) {
        // todo: send all will not work here,
        // subscription is always about a graph
        this.sendAll('network', 'error', {
          message: err.toString()
        });
      });
    }
    return _this;
  }

  _createClass(WebSocketTransport, [{
    key: 'noCircularJSON',
    value: function noCircularJSON(data) {
      return JSON.stringify(data, function (key, val) {
        if (val && (typeof val === 'undefined' ? 'undefined' : _typeof(val)) === 'object' && !Array.isArray(val) && val.constructor && val.constructor.name !== 'Object') {
          return '[' + val.constructor.name + ']';
        }
        return val;
      });
    }

    /**
     *
     * Send the response.
     *
     */

  }, {
    key: '$send',
    value: function $send(data, conn) {
      if (data.command === 'error') {
        if (data.payload.message && data.payload.message.message) {
          data.payload.message = data.payload.message.message;
        }
      }
      conn.sendUTF(this.noCircularJSON(data));
    }
  }]);

  return WebSocketTransport;
}(BaseTransport);

module.exports = WebSocketTransport;