'use strict';

/* global WebSocket */

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var RWebSocket = require('../../vendor/reconnecting-websocket/reconnecting-websocket');
var BaseTransport = require('./base');

/**
*
* Websocket Client Transport to be used in the browser
*
* Usage:
*
*  const nm = new NetworkMonitor({
*    transport: new WebSocketBrowserTransport({ logger: console }),
*    logger: console
*  })
*  nm.monitor(ioHandler)
*
*  Note: sendAll just keeps on working although we are always
*        just one connection, we could register to several server though.
*
* @params {Object} options Options
* @params {String} options.host ws hostname
* @params {String} options.port ws port
* @params {String} options.mount ws `mount` point
* @params {String} options.protocol noflo
* @params {String} options.secure
*/

var WebSocketBrowserTransport = function (_BaseTransport) {
  _inherits(WebSocketBrowserTransport, _BaseTransport);

  function WebSocketBrowserTransport(options) {
    _classCallCheck(this, WebSocketBrowserTransport);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(WebSocketBrowserTransport).call(this, options));

    var self = _this;

    _this.connected = false;

    if (!options.host) {
      throw Error('Must provide a hostname');
    }

    if (!options.port) {
      throw Error('Must provide a portname');
    }

    options.protocol = options.protocol || 'noflo';

    _this.options = options;

    var url = [options.secure ? 'wss://' : 'ws://', options.host, ':', options.port, '/', options.mount || ''].join('');

    RWebSocket.debugAll = true;

    var conn = _this.conn = new RWebSocket(url, options.protocol);
    conn.logger = _this.logger;
    conn.reconnectInterval = 1000;
    conn.reconnectDecay = 1.5;
    conn.reconnectAttempts = 0;
    conn.timeoutInterval = 2000;

    conn.onopen = function onOpen() {
      self.logger.info('wsClient', 'WebSocket client connected', conn.protocol);
      self.connected = true;
      self.emit('connected');

      // self.register(conn)

      conn.onmessage = function (event) {
        var payload = JSON.parse(event.data);
        self.receive(payload, conn);
      };

      conn.onerror = function (error) {
        self.logger.error('wsClient', 'Connection Error: ', error);
      };

      conn.onclose = function (event) {
        self.logger.info('wsClient', 'connection close', event.data);
        self.connected = false;
        self.emit('disconnected');
        // self.unregister(conn)
      };
    };
    return _this;
  }

  /**
   *
   * Send the response.
   *
   */


  _createClass(WebSocketBrowserTransport, [{
    key: '$send',
    value: function $send(data /* , conn */) {
      if (this.conn.readyState === WebSocket.OPEN) {
        this.setSecret(data);
        this.conn.send(JSON.stringify(data));
      } else {
        this.logger.warn('WebSocketBrowserTransport:', 'connection already closed');
      }
    }
  }]);

  return WebSocketBrowserTransport;
}(BaseTransport);

module.exports = WebSocketBrowserTransport;