'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var WebSocketClient = require('websocket/lib/WebSocketClient');
var BaseTransport = require('./base');

/**
*
* Websocket Client Transport to be used on the server
*
* Usage:
*
*  const nm = new NetworkMonitor({
*    transport: new WebSocketClientTransport({ logger: console }),
*    logger: console
*  })
*  nm.monitor(ioHandler)
*
*  Note: sendAll just keeps on working although we are always
*        just one connection, we could register to several server though.
*
* @params {Object} options Options
* @params {String} options.host ws hostname
* @params {String} options.port ws port
* @params {String} options.mount ws `mount` point
* @params {String} options.protocol noflo
*/

var WebSocketClientTransport = function (_BaseTransport) {
  _inherits(WebSocketClientTransport, _BaseTransport);

  function WebSocketClientTransport(options) {
    _classCallCheck(this, WebSocketClientTransport);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(WebSocketClientTransport).call(this, options));

    var self = _this;

    if (!options.host) {
      throw Error('Must provide a hostname');
    }

    if (!options.port) {
      throw Error('Must provide a portname');
    }

    options.protocol = options.protocol || 'noflo';

    _this.options = options;

    var url = [options.secure ? 'wss://' : 'ws://', options.host, ':', options.port, '/', options.mount || ''].join('');

    _this.ws = new WebSocketClient();
    _this.ws.on('connect', function (conn) {
      self.logger.info('wsClient', 'WebSocket client connected', conn.protocol);

      // self.register(conn)

      conn.on('message', function (message) {
        if (message.type === 'utf8') {
          var payload = JSON.parse(message.utf8Data);
          self.receive(payload, conn);
        } else {
          self.logger.warn('wsClient', 'unknown message type', message);
        }
      });

      conn.on('error', function (error) {
        self.logger.error('Connection Error: ' + error.toString());
      });

      conn.on('close', function (message) {
        self.logger.info('wsClient', 'connection close', message);

        // self.unregister(conn)
      });
    });

    _this.ws.on('connectFailed', function (error) {
      self.logger.warn('wsClient', 'Connect Error: ' + error.toString());
    });

    _this.ws.connect(url, options.protocol);
    return _this;
  }

  /**
   *
   * Send the response.
   *
   */


  _createClass(WebSocketClientTransport, [{
    key: '$send',
    value: function $send(data, conn) {
      this.setSecret(data);
      conn.sendUTF(JSON.stringify(data));
    }
  }]);

  return WebSocketClientTransport;
}(BaseTransport);

module.exports = WebSocketClientTransport;