'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var websocket = require('websocket');
var util = require('util');
var BaseTransport = require('./base');
var printmsg = require('../util').printmsg;

var WebSocketServer = websocket.server;
var WebSocketRouter = websocket.router;

/**
*
* Websocket Transport Proxy
*
* Can be used standalone.
*
* The proxied result is available at /proxy
*
* UI talks to /
*
* Client who want's to proxy talks to /proxy
*
* Usage:
*
*  new WebSocketProxyTransport({ logger: console }),
*
* @params {Server} http HTTP Server
* @params {Object} options Options
* @params {String} options.catchExceptions Send all exceptions to UI.
* @params {String} options.type runtime type chix
*/

var WebSocketProxyTransport = function (_BaseTransport) {
  _inherits(WebSocketProxyTransport, _BaseTransport);

  function WebSocketProxyTransport(http, options) {
    _classCallCheck(this, WebSocketProxyTransport);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(WebSocketProxyTransport).call(this, options));

    var self = _this;

    _this.options = options;

    _this.ws = new WebSocketServer({
      httpServer: http
    });

    _this.router = new WebSocketRouter();
    _this.router.attachServer(_this.ws);

    // The UI talking
    _this.router.mount('*', 'noflo', function (req) {
      var conn = req.accept(req.origin);

      self.logger.info('wsProxyServer', 'connection accepted from', req.origin);

      self.register(conn, conn.protocol);

      // send to the other mountpoint subscribers
      conn.on('message', function (message) {
        if (message.type === 'utf8') {
          (function () {
            var data = JSON.parse(message.utf8Data);

            self.logger.debug('wsProxyServer', 'receveived %s:%s', data.protocol, data.command);

            var all = self.getAll('noflo-runtime');

            if (all.length === 0) {
              self.logger.debug('wsProxyServer', 'request ignored, client list empty');
            } else {
              // Send this info to all graphRunners
              all.forEach(function (client) {
                // there is info in the schema.
                printmsg(self.logger, data, '  → ', client.remoteAddress);
                self.send(data, client);
              });
            }

            // we don't receive.
            // self.receive(payload, conn)
          })();
        } else {
          self.logger.warn('wsProxyServer', 'unknown message type', conn.protocol, message);
        }
      });

      conn.on('close', function (message) {
        self.logger.info('wsProxyServer', util.format('connection from %s closed', conn.remoteAddress), message);

        self.unregister(conn, conn.protocol);
      });
    });

    // The Runtime within the browser talking
    _this.router.mount('*', 'noflo-runtime', function (req) {
      var conn = req.accept(req.origin);

      // Graph Runners.
      self.register(conn, conn.protocol);

      self.logger.info('wsProxyServer', 'connection accepted from', conn.remoteAddress);

      // send to the other mountpoint subscribers
      // This will work, but with multiple it will not ofcourse.
      // Anyway let's just send every message from every client back.
      conn.on('message', function (message) {
        if (message.type === 'utf8') {
          (function () {
            var data = JSON.parse(message.utf8Data);

            // Send this info to all graphRunners
            self.getAll('noflo').forEach(function (client) {
              if (data.command !== 'component') {
                printmsg(self.logger, data, '←   ', client.remoteAddress);
              }
              self.send(data, client);
            });

            // we don't receive.
            // self.receive(payload, conn)
          })();
        } else {
          self.logger.warn('wsProxyServer', 'unknown message type', conn.protocol, message);
        }
      });

      conn.on('close', function (message) {
        self.logger.info('wsProxyServer', util.format('connection from %s closed', conn.remoteAddress), message);

        self.unregister(conn, conn.protocol);
      });
    });
    return _this;
  }

  /**
   *
   * Send the response.
   *
   */


  _createClass(WebSocketProxyTransport, [{
    key: '$send',
    value: function $send(data, conn) {
      conn.sendUTF(JSON.stringify(data));
    }
  }], [{
    key: 'create',
    value: function create(http, options) {
      return new WebSocketProxyTransport(http, options);
    }
  }]);

  return WebSocketProxyTransport;
}(BaseTransport);

module.exports = WebSocketProxyTransport;