'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BaseTransport = require('./base');
var WebSocketClient = require('websocket/lib/WebSocketClient');
var httpServer = require('http');
var printmsg = require('../util').printmsg;
var WebSocketServer = require('websocket').server;

function defaultConductor(transport) {
  return {
    send: function send(payload, clientConn /* , runtimeConn */) {
      return transport.send(payload, clientConn);
    },
    recv: function recv(payload, runtimeConn /* , clientConn */) {
      return transport.send(payload, runtimeConn);
    }
  };
}

/**
*
* Websocket Proxy Server Transport
*
* Will try to connect to a runtime server.
* If it succeeds will start a websocket server which will listen for incomming messages.
*
* Each message received will directly be send to the attached runtime.
*
* Any messages received from the runtime will be send back directly to our client (The UI)
*
* @params {Server} http HTTP Server
* @params {Object} options Options
* @params {String} options.url Websocket url runtime.
* @params {String} options.protocol Protocol noflo.
* @params {String} options.catchExceptions Send all exceptions to UI.
* @params {String} options.type runtime type chix
* @params {String} options.conductor Message conductor Send & Receive negotiator
* @params {HttpServer} http Http Server instance
*/

var WebSocketProxyServerTransport = function (_BaseTransport) {
  _inherits(WebSocketProxyServerTransport, _BaseTransport);

  function WebSocketProxyServerTransport(options, http) {
    _classCallCheck(this, WebSocketProxyServerTransport);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(WebSocketProxyServerTransport).call(this, options));

    var self = _this;

    if (!options.url) {
      throw Error('WebSocketProxyServerTransport: runtime url option required');
    }

    _this.conductor = options.conductor ? options.conductor(_this) : defaultConductor(_this);

    if (!http) {
      http = httpServer.createServer(function (req, res) {
        res.writeHead(200, {
          'Content-Type': 'application/json'
        });
        res.end(JSON.stringify({
          connections: _this.connections.length,
          pendingRequests: _this.pendingRequests.length
        }));
      });
      self.logger.info('wsServer', 'created http server');
    }

    options.protocol = options.protocol || 'noflo';

    _this.options = options;

    // start the client first, the one connecting to noflo-nodejs

    // cannot use this transport, because it will try to .receive()
    // so just use bare
    _this.client = new WebSocketClient();

    _this.conn = null;
    _this.serverConn = null;

    startServer();

    // there is always just one client
    _this.client.on('connect', function (conn) {
      self.conn = conn;

      self.logger.info('wsClient', 'WebSocket client connected', self.conn.remoteAddress);

      self.conn.on('message', function (message) {
        if (message.type === 'utf8') {
          var payload = JSON.parse(message.utf8Data);
          printmsg(self.logger, payload, '←   ', conn.remoteAddress);
          self.emit('send', payload, conn.remoteAddress);

          self.conductor.send(payload, self.serverConn, conn);
          // self.send(payload, self.serverConn)
        } else {
          self.logger.warn('wsClient', 'unknown message type', message);
        }
      });

      self.conn.on('error', function (error) {
        self.logger.error('Connection Error: ' + error.toString());
      });

      self.conn.on('close', function (message) {
        self.logger.info('wsClient', 'connection close', message);
        // self.unregister(conn)
      });
    });

    _this.client.on('connectFailed', function (error) {
      self.logger.warn('wsClient', 'Connect Error: ' + error.toString());
    });

    _this.client.connect(options.url, options.protocol);
    // need the send back part

    function startServer() {
      self.server = new WebSocketServer({
        httpServer: http
      });

      self.server.on('request', function (req) {
        var conn = void 0;
        try {
          conn = req.accept('noflo', req.origin);
          self.serverConn = conn;
          self.logger.info('wsServer', 'connection accepted', conn.remoteAddress);
        } catch (e) {
          console.log(e);
          self.logger.error('wsServer', e);
          return;
        }

        conn.on('message', function (message) {
          if (message.type === 'utf8') {
            var payload = JSON.parse(message.utf8Data);
            printmsg(self.logger, payload, '  → ', conn.remoteAddress);
            self.emit('recv', payload, conn.remoteAddress);
            self.conductor.recv(payload, self.conn, self.serverConn);
            // self.send(payload, self.conn)
          } else {
            self.logger.warn('wsServer', 'unknown message type', message);
          }
        });

        conn.on('close', function (message) {
          self.logger.info('wsServer', 'connection close', message);
        });
      });

      if (options.catchExceptions) {
        process.on('uncaughtException', function (err) {
          self.send({ protocol: 'network', command: 'error', message: err.toString() });
        });
      }
    }
    return _this;
  }

  _createClass(WebSocketProxyServerTransport, [{
    key: 'noCircularJSON',
    value: function noCircularJSON(data) {
      return JSON.stringify(data, function (key, val) {
        if (val && (typeof val === 'undefined' ? 'undefined' : _typeof(val)) === 'object' && !Array.isArray(val) && val.constructor && val.constructor.name !== 'Object') {
          return '[' + val.constructor.name + ']';
        }
        return val;
      });
    }

    /**
     *
     * Send the response.
     *
     */

  }, {
    key: 'send',
    value: function send(data, conn) {
      if (data.command === 'error') {
        if (data.payload.message && data.payload.message.message) {
          data.payload.message = data.payload.message.message;
        }
      }
      conn.sendUTF(this.noCircularJSON(data));
    }
  }], [{
    key: 'create',
    value: function create(http, options) {
      return new WebSocketProxyServerTransport(options, http);
    }
  }]);

  return WebSocketProxyServerTransport;
}(BaseTransport);

module.exports = WebSocketProxyServerTransport;